/*
 * Copyright 2010 Nathanaël Cottin - http://www.ezjson.org
 *
 * This file is part of EZ JSON (easy JSON integration).
 *
 * EZ JSON is free software: you can redistribute it and/or modify it under the
 * terms of the Lesser GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or any later version.
 *
 * EZ JSON is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Insane. If not, see <http://www.gnu.org/licenses/>.
 */
package org.ezjson;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;

/**
 * A JSON array is able to hold an ordered collection of JSON values,
 * independently from their actual type. This means that, according to the JSON
 * specification, an array can mix different JSON types.
 * <br/>Note that this array is iterable: each internal JSON value can be
 * accessed using a <tt>foreach</tt> construct
 *
 * @author ncottin
 */
public final class JSONArray implements JSONValue, Iterable<JSONValue> {

    private Collection<JSONValue> values;

    /**
     * Creates an empty JSON array
     */
    public JSONArray() {
        values = new LinkedList();
    }

    /**
     * Gets the size of this array. The initial size is <tt>0</tt>
     * @return The number of elements of this array
     */
    public int size() {
        return values.size();
    }

    /**
     * Gets the current collection of values this array holds
     *
     * @return The internal collection of values
     */
    public Collection<JSONValue> getValues() {
        return values;
    }

    /**
     * Ensures that this JSON array contains the specified value
     *
     * @param value A possibly <tt>null</tt> value to add to the current
     * collection
     */
    public void add(JSONValue value) {
        values.add(value);
    }

    public Iterator<JSONValue> iterator() {
        return values.iterator();
    }
}