/*
 * Copyright 2010 Nathanaël Cottin - http://www.ezjson.org
 *
 * This file is part of EZ JSON (easy JSON integration).
 *
 * EZ JSON is free software: you can redistribute it and/or modify it under the
 * terms of the Lesser GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or any later version.
 *
 * EZ JSON is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Insane. If not, see <http://www.gnu.org/licenses/>.
 */
package org.ezjson;

import java.io.BufferedReader;
import java.io.IOException;

/**
 * Default JSON objects serialized form parser
 * 
 * @author ncottin
 */
public final class JSONDecoder {

    /**
     * Generates a JSON object from a source stream
     * 
     * @param in The non-<tt>nul</tt> source reader which contains a JSON
     * encoded object
     * @return The decoded JSON object from the source reader
     * @throws IOException The decoder could not access the reader content
     * @throws JSONDecodingException The source content does not conform to
     * a valid JSON encoding
     */
    public static JSONObject decode(BufferedReader in) throws IOException, JSONDecodingException {
        skipSpaces(in);
        int r = in.read();
        if (r != '{') {
            throw new JSONDecodingException();
        }

        skipSpaces(in);
        return decodeObject(in);
    }

    private static JSONObject decodeObject(BufferedReader in) throws IOException, JSONDecodingException {
        JSONObject obj = new JSONObject();
        in.mark(1);
        int r = in.read();
        if (r == '}') {
            // Empty object
            return obj;
        }

        in.reset();
        do {
            decodeElement(in, obj);
            skipSpaces(in);
            r = in.read();
            if (r == ',') {
                skipSpaces(in);
            }
        } while (r == ',');

        if (r != '}') {
            throw new JSONDecodingException();
        }

        return obj;
    }

    private static void skipSpaces(BufferedReader in) throws IOException, JSONDecodingException {
        int r;
        do {
            in.mark(1);
            r = in.read();
        } while ((r != -1) && isSpace(r));

        if (r == -1) {
            throw new JSONDecodingException();
        }

        in.reset();
    }

    private static boolean isSpace(int c) {
        return (c == ' ') || (c == '\t') || (c == '\r') || (c == '\n');
    }

    private static void decodeElement(BufferedReader in, JSONObject parent) throws IOException, JSONDecodingException {
        String name = decodeName(in);
        skipSpaces(in);
        int r = in.read();
        if (r != ':') {
            throw new JSONDecodingException();
        }

        skipSpaces(in);
        parent.put(name, decodeValue(in));
    }

    private static String decodeName(BufferedReader in) throws IOException, JSONDecodingException {
        int r = in.read();
        if (r != '\"') {
            throw new JSONDecodingException();
        }

        StringBuilder sb = new StringBuilder();
        while ((r = in.read()) != '\"') {
            if (r == -1) {
                throw new JSONDecodingException();
            }

            sb.append((char)r);
        }

        return sb.toString();
    }

    private static JSONValue decodeValue(BufferedReader in) throws IOException, JSONDecodingException {
        int r = in.read();
        if (r == '{') {
            // Decode an object
            skipSpaces(in);
            return decodeObject(in);
        }

        if (r == '[') {
            // Decode an array
            JSONArray res = new JSONArray();
            skipSpaces(in);
            in.mark(1);
            r = in.read();
            if (r == ']') {
                // Empty array
                return res;
            }

            in.reset();
            do {
                res.add(decodeValue(in));
                skipSpaces(in);
                r = in.read();
                if (r == ',') {
                    skipSpaces(in);
                }
            } while (r == ',');

            if (r != ']') {
                throw new JSONDecodingException();
            }

            return res;
        }

        
        if (r == '\"') {
            return new JSONString(decodeString(in));
        }

        StringBuilder sb = new StringBuilder();
        if (r == 'n') {
            // Try to decode "null"
            for (int i=0; i<3; ++i) {
                sb.append((char)in.read());
            }

            if (!"ull".equals(sb.toString())) {
                throw new JSONDecodingException();
            }

            return null;
        }

        if (r == 't') {
            // Try to decode boolean "true"
            for (int i=0; i<3; ++i) {
                sb.append((char)in.read());
            }

            if (!"rue".equals(sb.toString())) {
                throw new JSONDecodingException();
            }

            return new JSONBoolean(true);
        }

        if (r == 'f') {
            // Try to decode boolean "false"
            for (int i=0; i<4; ++i) {
                sb.append((char)in.read());
            }

            if (!"alse".equals(sb.toString())) {
                throw new JSONDecodingException();
            }

            return new JSONBoolean(false);
        }
        
        // Try to decode a number
        if (!isDigit(r)) {
            throw new JSONDecodingException();
        }

        sb.append((char)r);
        return readNumber(sb, in);
    }

    private static String decodeString(BufferedReader in) throws IOException, JSONDecodingException {
        StringBuilder sb = new StringBuilder();
        int r;
        while ((r = in.read()) != '\"') {
            if (r == -1) {
                throw new JSONDecodingException();
            }

            if (r == '\\') {
                sb.append(decodeProtected(in));
            } else {
                sb.append((char)r);
            }
        }

        return sb.toString();
    }

    private static char decodeProtected(BufferedReader in) throws IOException, JSONDecodingException {
        int r = in.read();
        if (r == -1) {
            throw new JSONDecodingException();
        }

        if (r == '\\') return '\\';
        if (r == 'b') return '\b';
        if (r == 'f') return '\f';
        if (r == 'r') return '\r';
        if (r == 'n') return '\n';
        if (r == 't') return '\t';
        return (char)r;
    }
    
    private static boolean isDigit(int r) {
        return (r == '-') || ((r >= '0') && (r <= '9'));
    }

    private static JSONNumber readNumber(StringBuilder sb, BufferedReader in) throws IOException, JSONDecodingException {
        int nbRead = 0;
        int r;
        for (;;) {
            in.mark(1);
            r = in.read();
            if (r == -1) {
                throw new JSONDecodingException();
            }

            if (isSeparator(r)) {
                String value = sb.toString();
                in.reset();
                try {
                    int val = Integer.parseInt(value);
                    return new JSONInteger(val);
                } catch (NumberFormatException e) {
                    try {
                        double val = Double.parseDouble(value);
                        return new JSONReal(val);
                    } catch (NumberFormatException e2) {
                        throw new JSONDecodingException();
                    }
                }
            }

            sb.append((char)r);
            ++nbRead;
        }
    }

    private static boolean isSeparator(int r) {
        return (r == ',') || (r == '}') || (r == ']') || isSpace(r);
    }

    /**
     * Attempts to get a JSON boolean from any JSON value (usually a JSON
     * string) when the actual type of this JSON value is unknown. This
     * operation is useful when a decoded JSON object may hold JSON strings
     * instead of JSON booleans (i&#46;e&#46; when encoded boolean values are
     * represented between double quotes, this decoder decodes as JSON strings).
     * Note that this operation is not required when the default encoder is
     * used...
     *
     * @param value A possibly <tt>null</tt> value (mostly obtained from a
     * decoded JSON object)
     * @return The given value if this value is a JSON boolean or <tt>null</tt>,
     * or the corresponding JSON boolean, which equals <tt>true</tt> only when
     * the provided value is a JSON string which equals <tt>"true"</tt>
     * @throws JSONDecodingException The non-<tt>null</tt> provided value does
     * not represent a boolean value
     */
    public static JSONBoolean getAsBoolean(JSONValue value) throws JSONDecodingException {
        if ((value == null) || (value instanceof JSONBoolean)) {
            return (JSONBoolean)value;
        }

        if (value instanceof JSONString) {
            String val = ((JSONString)value).getValue();
            return new JSONBoolean("true".equals(val));
        }

        throw new JSONDecodingException();
    }

    /**
     * Attempts to get a JSON number from any JSON value (usually a JSON
     * string) when the actual type of this JSON value is unknown. This
     * operation is useful when a decoded JSON object may hold JSON strings
     * instead of JSON numbers (i&#46;e&#46; when encoded numbers are
     * represented between double quotes, this decoder decodes as JSON strings).
     * Note that this operation is not required when the default encoder is
     * used...
     *
     * @param value A possibly <tt>null</tt> value (mostly obtained from a
     * decoded JSON object)
     * @return The corresponding JSON number (i&#46;e&#46; integer or real) or
     * <tt>null</tt> in case the provided value is <tt>null</tt>. The actual
     * type can be determined by means of {@link JSONNumber#isInteger()}
     * @throws JSONDecodingException The non-<tt>null</tt> provided value does
     * not represent a number value
     */
    public static JSONNumber getAsNumber(JSONValue value) throws JSONDecodingException {
        if (value == null) {
            return null;
        }

        if (value instanceof JSONNumber) {
            return (JSONNumber)value;
        }

        if (value instanceof JSONString) {
            String val = ((JSONString)value).getValue();
            try {
                int v = Integer.parseInt(val);
                return new JSONInteger(v);
            } catch (NumberFormatException e) {
                try {
                    double v = Double.parseDouble(val);
                    return new JSONReal(v);
                } catch (NumberFormatException e2) {
                }
            }
        }

        throw new JSONDecodingException();
    }

    /**
     * Attempts to get a JSON integer from any JSON value (usually a JSON
     * string) when the actual type of this JSON value is unknown. This
     * operation is useful when a decoded JSON object may hold JSON strings
     * instead of JSON numbers (i&#46;e&#46; when encoded integers are
     * represented between double quotes, this decoder decodes as JSON strings).
     * Note that this operation is not required when the default encoder is
     * used...
     *
     * @param value A possibly <tt>null</tt> value (mostly obtained from a
     * decoded JSON object)
     * @return The corresponding JSON integer or <tt>null</tt> in case the
     * provided value is <tt>null</tt>
     * @throws JSONDecodingException The non-<tt>null</tt> provided value does
     * not represent an integer value
     */
    public static JSONInteger getAsInteger(JSONValue value) throws JSONDecodingException {
        if ((value == null) || (value instanceof JSONInteger)) {
            return (JSONInteger)value;
        }

        if (value instanceof JSONString) {
            String val = ((JSONString)value).getValue();
            try {
                int v = Integer.parseInt(val);
                return new JSONInteger(v);
            } catch (NumberFormatException e2) {
            }
        }

        throw new JSONDecodingException();
    }

    /**
     * Attempts to get a JSON real from any JSON value (usually a JSON
     * string or a JSON integer) when the actual type of this JSON value is
     * unknown. This operation is useful when a decoded JSON object may hold
     * JSON strings or integers instead of JSON reals (i&#46;e&#46; when encoded
     * reals are represented by strings - between double quotes - or by integer
     * values), this decoder respectively decodes as JSON strings or JSON
     * integer). Note that this operation is not required when the default
     * encoder is used...
     *
     * @param value A possibly <tt>null</tt> value (mostly obtained from a
     * decoded JSON object)
     * @return The corresponding JSON real or <tt>null</tt> in case the
     * provided value is <tt>null</tt>
     * @throws JSONDecodingException The non-<tt>null</tt> provided value does
     * not represent a real value
     */
    public static JSONReal getAsReal(JSONValue value) throws JSONDecodingException {
        if ((value == null) || (value instanceof JSONReal)) {
            return (JSONReal)value;
        }

        if (value instanceof JSONInteger) {
            double val = (double)((JSONInteger)value).getValue();
            return new JSONReal(val);
        }

        if (value instanceof JSONString) {
            String val = ((JSONString)value).getValue();
            try {
                double v = Double.parseDouble(val);
                return new JSONReal(v);
            } catch (NumberFormatException e2) {
            }
        }

        throw new JSONDecodingException();
    }
}
