/*
 * Copyright 2010 Nathanaël Cottin - http://www.ezjson.org
 *
 * This file is part of EZ JSON (easy JSON integration).
 *
 * EZ JSON is free software: you can redistribute it and/or modify it under the
 * terms of the Lesser GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or any later version.
 *
 * EZ JSON is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Insane. If not, see <http://www.gnu.org/licenses/>.
 */
package org.ezjson;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Iterator;

/**
 * Default JSON encoder to serialize JSON objects
 * 
 * @author ncottin
 */
public final class JSONEncoder {

    /**
     * Indentation for a pretty encoding using a space character
     */
    public static final char SPACE = ' ';

    /**
     * Indentation for a pretty encoding using a tabulation character
     */
    public static final char TABULATION = '\t';

    /**
     * Serializes a JSON object to a destination writer using the most
     * compact format. This encoding is primarily designed for
     * machine-to-machine communications and may not be easily readable
     * 
     * @param obj The non-<tt>null</tt> source JSON object to serialize
     * @param out The destination writer to receive the serialized form of the
     * provided JSON object
     * @throws IOException The destination writer could not be written
     * @see #encodePretty(org.ezjson.JSONObject, char, java.io.PrintWriter)
     */
    public static void encode(JSONObject obj, PrintWriter out) throws IOException {
        out.print('{');
        String name;
        JSONValue value;
        Iterator<String> it = obj.iterator();
        while (it.hasNext()) {
            name = it.next();
            out.print('\"');
            out.print(name);
            out.print("\":");
            value = obj.get(name);
            encodeGenericValue(value, out);
            if (it.hasNext()) {
                out.print(',');
            }
        }

        out.print('}');
        out.flush();
    }

    private static void encodeGenericValue(JSONValue value, PrintWriter out) throws IOException {
        if (value == null) {
            out.print("null");
            return;
        }

        if (value instanceof JSONObject) {
            encode((JSONObject)value, out);
            return;
        }

        if (value instanceof JSONArray) {
            encodeValue((JSONArray)value, out);
            return;
        }

        if (value instanceof JSONString) {
            encodeValue((JSONString)value, out);
            return;
        }

        if (value instanceof JSONBoolean) {
            encodeValue((JSONBoolean)value, out);
            return;
        }

        if (value instanceof JSONInteger) {
            encodeValue((JSONInteger)value, out);
            return;
        }

        if (value instanceof JSONReal) {
            encodeValue((JSONReal)value, out);
            return;
        }
    }

    private static void encodeValue(JSONArray value, PrintWriter out) throws IOException {
        out.print('[');
        Iterator<JSONValue> val = value.iterator();
        while (val.hasNext()) {
            encodeGenericValue(val.next(), out);
            if (val.hasNext()) {
                out.print(',');
            }
        }

        out.print(']');
    }

    private static void encodeValue(JSONString value, PrintWriter out) throws IOException {
        String val = value.getValue();
        if (val == null) {
            out.print("null");
            return;
        }

        out.print('\"');
        print(val, out);
        out.print('\"');
    }

    private static void print(String value, PrintWriter out) throws IOException {
        // Any special char must be protected with reverse solidus
        for (char c : value.toCharArray()) {
            if (isProtected(c)) {
                out.print('\\');
            }

            out.print(c);
        }
    }

    private static boolean isProtected(char c) {
        return (c == '\\') || (c == '\"');
    }

    private static void encodeValue(JSONBoolean value, PrintWriter out) throws IOException {
        out.print(value.getValue());
    }

    private static void encodeValue(JSONInteger value, PrintWriter out) throws IOException {
        out.print(value.getValue());
    }

    private static void encodeValue(JSONReal value, PrintWriter out) throws IOException {
        out.print(value.getValue());
    }

    /**
     * Serializes a JSON object using a human-readble encoding (which includes
     * a pretty indentation) with no initial indentation
     *
     * @param obj The non-<tt>null</tt> source JSON object to serialize
     * @param spacer A space character used for indentation, among
     * {@link #SPACE} and {@link #TABULATION}
     * @param out The destination writer to receive the serialized form of the
     * provided JSON object
     * @throws IOException Writing to the destination writer failed
     */
    public static void encodePretty(JSONObject obj, char spacer, PrintWriter out) throws IOException {
        encodePretty(obj, 0, spacer, out);
    }

    /**
     * Serializes a JSON object using a human-readble encoding (which includes
     * a pretty indentation) with an initial indentation. This initial
     * indentation corresponds to the root JSON object indentation
     *
     * @param obj
     * @param offset The initial indentation expressed with a positive number
     * of space characters
     * @param spacer A space character used for indentation, among
     * {@link #SPACE} and {@link #TABULATION}
     * @param out out The destination writer to receive the serialized form of the
     * provided JSON object
     * @throws IOException Writing to the destination writer failed
     */
    public static void encodePretty(JSONObject obj, int offset, char spacer, PrintWriter out) throws IOException {
        printOffset(offset, spacer, out);
        encodePrettyValue(obj, offset, spacer, out);
    }

    private static void printOffset(int offset, char spacer, PrintWriter out) throws IOException {
        for (int i=0; i < offset; ++i) {
            out.print(spacer);
        }
    }

    private static void encodeGenericPrettyValue(boolean inArray, JSONValue value, int offset, char spacer, PrintWriter out) throws IOException {
        if (inArray) {
            printOffset(offset, spacer, out);
        }

        if (value == null) {
            out.print("null");
            return;
        }

        if (value instanceof JSONObject) {
            encodePrettyValue((JSONObject)value, offset, spacer, out);
            return;
        }

        if (value instanceof JSONArray) {
            encodePrettyValue((JSONArray)value, offset, spacer, out);
            return;
        }

        if (value instanceof JSONString) {
            encodeValue((JSONString)value, out);
            return;
        }

        if (value instanceof JSONBoolean) {
            encodeValue((JSONBoolean)value, out);
            return;
        }

        if (value instanceof JSONInteger) {
            encodeValue((JSONInteger)value, out);
            return;
        }

        if (value instanceof JSONReal) {
            encodeValue((JSONReal)value, out);
            return;
        }
    }

    private static void encodePrettyValue(JSONObject obj, int offset, char spacer, PrintWriter out) throws IOException {
        out.println('{');
        String name;
        JSONValue value;
        int nextOffset = offset + 1;
        Iterator<String> it = obj.iterator();
        boolean hasNext = it.hasNext();
        while (hasNext) {
            printOffset(nextOffset, spacer, out);
            name = it.next();
            out.print('\"');
            out.print(name);
            out.print("\":");
            value = obj.get(name);
            encodeGenericPrettyValue(false, value, nextOffset, spacer, out);
            hasNext = it.hasNext();
            if (hasNext) {
                out.print(',');
            }

            out.println();
        }

        printOffset(offset, spacer, out);
        out.print('}');
        out.flush();
    }

    private static void encodePrettyValue(JSONArray value, int offset, char spacer, PrintWriter out) throws IOException {
        out.println('[');
        int nextOffset = offset + 1;
        Iterator<JSONValue> val = value.iterator();
        while (val.hasNext()) {
            encodeGenericPrettyValue(true, val.next(), nextOffset, spacer, out);
            if (val.hasNext()) {
                out.print(',');
            }

            out.println();
        }

        printOffset(offset, spacer, out);
        out.print(']');
        out.flush();
    }
}
