/*
 * Copyright 2010 Nathanaël Cottin - http://www.ezjson.org
 *
 * This file is part of EZ JSON (easy JSON integration).
 *
 * EZ JSON is free software: you can redistribute it and/or modify it under the
 * terms of the Lesser GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or any later version.
 *
 * EZ JSON is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Insane. If not, see <http://www.gnu.org/licenses/>.
 */
package org.ezjson;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * A JSON object is a (possibly empty) set of name/value pairs.
 * <br/>Note that this JSON object can be iterated over its internal names
 * using a <tt>foreach</tt> construct
 * 
 * @author ncottin
 */
public final class JSONObject implements JSONValue, Iterable<String> {

    private Map<String, JSONValue> components;

    public JSONObject() {
        components = new HashMap();
    }

    /**
     * Makes sure that this JSON object holds a name/value pair whose name
     * equals the provided name
     *
     * @param name A non-<tt>null</tt> and non-empty name
     * @return <tt>true</tt> only when this JSON object holds a name/value
     * pair identified by the provided name
     */
    public boolean contains(String name) {
        return components.containsKey(name);
    }

    /**
     * Adds a name/value pair to this JSON object. Any existing pair with the
     * same name is replaced by the provided pair
     * 
     * @param name The non-<tt>null</tt> and non-empty name which identifies
     * the corresponding value
     * @param value A possibly <tt>null</tt> value
     * @return The previous value associated with the provided name, or
     * <tt>null</tt> if there was no mapping for the provided name
     */
    public JSONValue put(String name, JSONValue value) {
        return components.put(name, value);
    }

    /**
     *
     * @param name A non-<tt>null</tt> and non-empty name
     * @return The corresponding value when the provided name is registered, or
     * <tt>null</tt> if no name/value pair identified by the provided name is
     * found
     */
    public JSONValue get(String name) {
        return components.get(name);
    }

    public Iterator<String> iterator() {
        return components.keySet().iterator();
    }
}